/*  Copyright (c) 2002, 2003, 2006 The Regents of the University of Michigan.
 *  All rights reserved.
 *
 *  Andy Adamson <andros@citi.umich.edu>
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. Neither the name of the University nor the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED
 *  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *  DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 *  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 *  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 *  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
 *  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "libacl_nfs4.h"

/*
 * nfs4_insert_file_aces - read ACE entries from spec file into struct nfs4_acl
 *
 * inputs
 *	fd: spec_file holding ACL created by nfs4_getfacl and modified by user.
 *	aclp: holds the modified ACL to be passed to setxattr.
 * output
 *	0 on success OR -1 on error.
 *
 * XXX: fgets ace_buf of size 128 to hold on ACE. is this big enough?
 */
int nfs4_insert_file_aces(struct nfs4_acl *acl, FILE* fd, unsigned int index)
{
	char ace_buf[NFS4_MAX_ACESIZE];
	int is_dir;
	struct nfs4_ace *ace;

	is_dir = acl->is_directory;
	memset(ace_buf, 0, NFS4_MAX_ACESIZE);

	/* grab each ACE, one per line in fd */
	while (fgets(ace_buf, NFS4_MAX_ACESIZE, fd) != NULL) {
		/* process each ACE */
		if ((ace = nfs4_ace_from_string(ace_buf, is_dir)) == NULL)
			return -1;
		
		if (nfs4_insert_ace_at(acl, ace, index++)) {
			free(ace);
			return -1;
		}
		memset(ace_buf, 0, NFS4_MAX_ACESIZE);
	}
	if (acl->naces == 0)
		return -1;

	return 0;
}
